#pragma MESSAGE DISABLE C4002 /* WARNING C4002: Result not used is ignored */

#include "kb.h"
#include "LEDIO.h"
#include "display.h"
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"
#include "Cpu.h"
#include "display.h"
#include "state.h"

/* Global variables */
volatile byte CCR_reg;                 /* Current CCR register */


// ISR for unhandled vectors
ISR(Cpu_Interrupt)
{
}


/*
** ===================================================================
**     Method      :  _EntryPoint (bean MC9S08QE8_16)
**
**     Description :
**         Initializes the whole system like timing and so on. At the end 
**         of this function, the C startup is invoked to initialize stack,
**         memory areas and so on.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
extern void _Startup(void);            /* Forward declaration of external startup function declared in file Start12.c */

#pragma NO_FRAME
#pragma NO_EXIT
void _EntryPoint(void)
{
  /* SOPT1: COPE=0,COPT=1,STOPE=0,??=0,??=0,??=0,BKGDPE=1,RSTPE=0 */
  setReg8(SOPT1, 0x42);                 
  /* SPMSC1: LVDF=0,LVDACK=0,LVDIE=0,LVDRE=1,LVDSE=1,LVDE=1,??=0,BGBE=0 */
  setReg8(SPMSC1, 0x1C);                
  /* SPMSC2: LPR=0,LPRS=0,LPWUI=0,??=0,PPDF=0,PPDACK=0,PPDE=1,PPDC=0 */
  setReg8(SPMSC2, 0x02);                
  /* SPMSC3: LVWIE=0 */
  clrReg8Bits(SPMSC3, 0x08);            

  // Clock initialization.  We're going for FLL Engaged Interal (FEI); 
  /* ICSC1: CLKS=0,RDIV=0,IREFS=1,IRCLKEN=0,IREFSTEN=0 */
  setReg8(ICSC1, 0x04);                /* Initialization of the ICS control register 1 */ 
  /* ICSC2: BDIV=1,RANGE=0,HGO=0,LP=0,EREFS=0,ERCLKEN=0,EREFSTEN=0 */
  setReg8(ICSC2, 0x40);                /* Initialization of the ICS control register 2 */ 
  while(!ICSSC_IREFST) {               /* Wait until the source of reference clock is internal clock */
  }
  /* ICSSC: DRST_DRS=0,DMX32=0 */
  clrReg8Bits(ICSSC, 0xE0);            /* Initialization of the ICS status and control */ 
  while((ICSSC & 0xC0) != 0x00) {      /* Wait until the FLL switches to Low range DCO mode */
  }

  __asm   jmp _Startup ;               /* Jump to C startup code */

}
void main(void)
{
    //FCDIV = 0x4c;
    //FCDIV = 0x29;
    //FCDIV = 0x27;
    //FCDIV = 0x13;
    //FCDIV = 0x4;
    /* PTASE: PTASE4=1,PTASE3=1,PTASE2=1,PTASE1=1,PTASE0=1 */
    //setReg8Bits(PTASE, 0x1F);             
    /* PTBSE: PTBSE7=1,PTBSE6=1,PTBSE5=1,PTBSE4=1,PTBSE3=1,PTBSE2=1,PTBSE1=1,PTBSE0=1 */
    //setReg8(PTBSE, 0xFF);                 
    /* PTADS: PTADS7=0,PTADS6=0,PTADS5=0,PTADS4=0,PTADS3=0,PTADS2=0,PTADS1=0,PTADS0=0 */
    //setReg8(PTADS, 0x00);                 
    /* PTBDS: PTBDS7=0,PTBDS6=0,PTBDS5=0,PTBDS4=0,PTBDS3=0,PTBDS2=0,PTBDS1=0,PTBDS0=0 */
    //setReg8(PTBDS, 0x00);                 
    /* SCGC1: TPM2=1,TPM1=1,ADC=1,IIC=1,SCI=1 */
    setReg8Bits(SCGC1, 0x75);             
    /* SCGC2: DBG=1,FLS=1,IRQ=1,KBI=1,ACMP=0,RTC=1,UNK=0, SPI=0 */
    setReg8Bits(SCGC2, 0xF4);             
    
    // set up rtc  (RTCPS = 0xb)
    RTCSC = (RTCSC & 0xf0) | 0xb; 
    RTCMOD = 0;
    RTCSC |= 0x10;        // set RTIE

    // set up gpio
    PTADD |= ((1 << 7) | (1 << 6));
    PTBDD |= ((1 << 7) | (1 << 6) | (1 << 5) | (1 << 3) | (1 << 2) | (1 << 1) | (1 << 0));
    PTCDD |= ((1 << 3) | (1 << 2) | (1 << 1) | (1 << 0));
    PTDDD |= ((1 << 3) | (1 << 2) | (1 << 0)); 
    LEDTimer_Init();

    // set up keyboard according to the order described in section 7.4.4
    PTAPE |= (0xf);
    KBISC = 0;
    KBIPE = (BUTTON0_KB | BUTTON1_KB | BUTTON2_KB | BUTTON3_KB);
    KBIES = 0;
    KBISC_KBACK = 1;
    // KBF = 0, KBACK = 0, KBIE = 1, KBIMOD = 0
    KBISC_KBIE = 1;

    stateshift(STATE_COUNTDOWN);
    __asm cli;      // enable interrupts

    for(;;) {
        if(displaytimerflag) {
            display_timer();
            displaytimerflag = 0;
        }
    }
} 

/* NVPROT: FPS7=1,FPS6=1,FPS5=1,FPS4=1,FPS3=1,FPS2=1,FPS1=1,FPDIS=1 */
const unsigned char NVPROT_INIT @0x0000FFBD = 0xFF;

/* NVOPT: KEYEN=0,FNORED=1,??=1,??=1,??=1,??=1,SEC01=1,SEC00=0 */
const unsigned char NVOPT_INIT @0x0000FFBF = 0x7E;

