#include "Cpu.h"
#include "time.h"
#include "state.h"
#include "display.h"
#include "PE_Error.h"
#pragma MESSAGE DISABLE C2705          /* WARNING C2705: Possible loss of data */
#pragma MESSAGE DISABLE C5919          /* WARNING C5919: Conversion of floating to unsigned integral */
#pragma MESSAGE DISABLE C5703          /* WARNING C5703: Parameter X declared in function F but not referenced */
#pragma MESSAGE DISABLE C4002          /* Disable warning C4002 "Result not used" */

char curbuf = 0;
unsigned short fbuf[10];
unsigned char highpower = 0, displaytimerflag = 0;

void LEDTimer_Init(void)
{
  /* TPM1SC: TOF=0,TOIE=0,CPWMS=0,CLKSB=0,CLKSA=0,PS2=0,PS1=0,PS0=0 */
  setReg8(TPM1SC, 0x00);               /* Stop HW; disable overflow interrupt and set prescaler to 0 */ 
  /* TPM1C0SC: CH0F=0,CH0IE=1,MS0B=0,MS0A=1,ELS0B=0,ELS0A=0,??=0,??=0 */
  setReg8(TPM1C0SC, 0x50);             /* Set output compare mode and enable compare interrupt */ 
  TPM1C0V = TPM1MOD = (word)0x1900;
  /* TPM1CNTH: BIT15=0,BIT14=0,BIT13=0,BIT12=0,BIT11=0,BIT10=0,BIT9=0,BIT8=0 */
  setReg8(TPM1CNTH, 0x00);             /* Reset HW Counter */ 
  /* TPM1SC: TOF=0,TOIE=0,CPWMS=0,CLKSB=0,CLKSA=1,PS2=0,PS1=0,PS0=0 */
  setReg8(TPM1SC, 0x08);               /* Set prescaler and run counter */ 
//  /* TPM1SC: TOF=0,TOIE=0,CPWMS=0,CLKSB=0,CLKSA=1,PS2=1,PS1=1,PS0=1 */
//  setReg8(TPM1SC, 0xf);               /* Set prescaler and run counter */ 

//  /* TPM2SC: TOF=0, TOIE=0, CPWMS=0, CLKSB=0, CLKSA=0, PS2=0, PS1=0, PS0=0 */
//  setReg8(TPM2SC, 0x0);
//  /* 10 = falling edge */
//  /* TPM2C0SC: CH0F=0, CH0IE=1, MS0B=0, MS0A=0, ELS0B=1, ELS0A=0, pin1=0, pin0=0 */
//  setReg8(TPM2C0SC, 0x48);
//  /* TPM2C0SC: CH0F=0, CH0IE=1, MS0B=0, MS0A=1, ELS0B=0, ELS0A=0, pin1=0, pin0=0 */
//  setReg8(TPM2C0SC, 0x50);
//  /* TPM2C0SC: CH0F=0, CH0IE=0, MS0B=0, MS0A=1, ELS0B=0, ELS0A=0, pin1=0, pin0=0 */
//  setReg8(TPM2C0SC, 0x10);
//  TPM2C0V = 0;
//  TPM2MOD = 0xffff;
//  TPM2CNT = 0;
//  /* TPM2SC: TOF=0, TOIE=0, CPWMS=0, CLKSB=0, CLKSA=1, PS2=1, PS1=1, PS0=1 */   /* so 0xffff ~= 2s */
//  setReg8(TPM2SC, 0xf);
//  /* TPM2SC: TOF=0, TOIE=0, CPWMS=0, CLKSB=0, CLKSA=1, PS2=0, PS1=0, PS0=0 */   /* so 0xffff ~= 2s */
//  setReg8(TPM2SC, 0x8);
}


ISR(LEDTimer_Interrupt)
{
    if(TPM1C0SC & 0x80) {
        /* TPM1C0SC: CH0F=0 */
        clrReg8Bits(TPM1C0SC, 0x80);         /* Reset compare interrupt request flag */ 
    }
    //fbuf[2] = LETTER(VAL2CHAR((tm.seconds & 0xf0) >> 4));
    //fbuf[3] = LETTER(VAL2CHAR(tm.seconds & 0xf));

    curbuf++;
    if(curbuf >= 10) {
        curbuf = 0;
    }
    do_display();
}

void do_display(void) {
// PTC1: 5
// PTC0: 0
// PTB3: 1
// PTB2: 6
// PTB1: 2
// PTB0: 7
// PTA7: 4
// PTA6: 3
// PTD3: 9
// PTD2: 8
    PTADD &= ~((1 << 6) | (1 << 7));
    PTBDD &= ~((1 << 3) | (1 << 2) | (1 << 1) | 1);
    PTCDD &= ~((1 << 1) | 1);
    PTDDD &= ~((1 << 3) | (1 << 2));
    if(highpower) {
        char idx = curbuf % 5;
        shift1(fbuf[idx] & 0xff);
        shift1((fbuf[idx] >> 8) & 0xff);
        PTBD_PTBD7 = 0;
        PTBD_PTBD7 = 1;
        shift2(fbuf[idx+5] & 0xff);
        shift2((fbuf[idx+5] >> 8) & 0xff);
        switch(idx) {
            case 0:
                PTCD &= ~(3);
                PTCDD |= 3;
                break;
            case 1:
                PTBD &= ~((1 << 3) | (1 << 2));
                PTBDD |= ((1 << 3) | (1 << 2));
                break;
            case 2:
                PTBD &= ~((1 << 1) | 1);
                PTBDD |= ((1 << 1) | 1);
                break;
            case 3: 
                PTAD &= ~(1 << 6);
                PTADD |= (1 << 6);
                PTDD &= ~(1 << 2);
                PTDDD |= (1 << 2);
                break;
            case 4: 
                PTAD &= ~(1 << 7);
                PTADD |= (1 << 7);
                PTDD &= ~(1 << 3);
                PTDDD |= (1 << 3);
                break;
        }
    } else {
        short s = fbuf[curbuf];
        if(s == 0) {
            return;
        }
        if(curbuf < 5) {
            //fbuf[curbuf] = (1 << (tm.seconds % 16));
            shift1(s & 0xff);
            shift1((s >> 8) & 0xff);
            PTBD_PTBD7 = 0;
            PTBD_PTBD7 = 1;
        } else {
            //fbuf[curbuf] = (1 << (tm.seconds % 16));
            //fbuf[curbuf] = (1 << 3) | (1 << 10);
            //fbuf[curbuf] = (1 << 3);
            shift2(s & 0xff);
            shift2((s >> 8) & 0xff);
        }
        switch(curbuf) {
            case 0:
                PTCD &= ~(1);
                PTCDD |= 1;
                break;
            case 1:
                PTBD &= ~(1 << 3);
                PTBDD |= (1 << 3);
                break;
            case 2:
                PTBD &= ~(1 << 1);
                PTBDD |= (1 << 1);
                break;
            case 3: 
                PTAD &= ~(1 << 6);
                PTADD |= (1 << 6);
                break;
            case 4: 
                PTAD &= ~(1 << 7);
                PTADD |= (1 << 7);
                break;
            case 5:
                PTCD &= ~(2); 
                PTCDD |= 2;
                break;
            case 6: 
                PTBD &= ~(1 << 2);
                PTBDD |= (1 << 2);
                break;
            case 7:
                PTBD &= ~(1);
                PTBDD |= 1;
                break;
            case 8:
                PTDD &= ~(1 << 2);
                PTDDD |= (1 << 2);
                break;
            case 9:
                PTDD &= ~(1 << 3);
                PTDDD |= (1 << 3);
                break;
        }
    }
//        switch(curbuf) {
//            case 3:
//                PTBD = 0x70;
//                PTBDD = 0x80;
//                //PTBD = 0x80;
//                //PTBDD = 0x70;
//                break;
//            case 2:
//                PTBD = 0xB0;
//                PTBDD = 0x40;
//                //PTBD = 0x40;
//                //PTBDD = 0xB0;
//                break;
//            case 1:
//                PTBD = 0xD0;
//                PTBDD = 0x20;
//                //PTBD = 0x20;
//                //PTBDD = 0xD0;
//                break;
//            case 0:
//                PTBD = 0xE0;
//                PTBDD = 0x10;
//                //PTBD = 0x10;
//                //PTBDD = 0xE0;
//                break;
//        }

}
void clearfbuf(void) {
    char i;
    for(i = 0; i < 10; i++) {
        fbuf[i] = 0;
    }
}

unsigned short x = 0;
unsigned char c = 1;
unsigned short nextrand(void) {
    x = ((5523 * x) + c) % 65536;
    return x;
}

// display timer.  called once per second
#define IN_COUNTDOWN() (curstate == STATE_COUNTDOWN)
#define PERIOD 100
unsigned char curclock = 0;
unsigned short bits = 0;
unsigned char final_message[10] = { 'N', 'I', 'N', 'J', 'A', 'P', 'A', 'R', 'T', 'Y' };
//unsigned char final_message[10] = { ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ' };
void display_timer(void) {
    unsigned char i;
    if(IN_COUNTDOWN()) {
        for(i = 0; i < 16; i++) {
            if(!(bits & (1 << i))) {
                fbuf[i] = LETTER(nextrand() % 26 + 'A');
            }
        }
    }
    curclock++;
    if(curclock >= (PERIOD)) {
        curclock = 0;

        if(bits == 0xffff) {
            bits = 0;
        } else {
            unsigned char nextbit;
            unsigned short mybits = bits;
            i = 0;
            for(nextbit = 0; nextbit < 16; nextbit++) {
                if(bits & (1 << nextbit)) {
                    i++;
                }
            }
            nextbit = nextrand() % (16-i);
            for(i = 0; i < 16; i++) {
                short x = (1 << i);
                if(bits & x) {
                    continue;
                }
                if(nextbit > 0) {
                    nextbit--;
                } else {
                    bits |= x;
                    if(IN_COUNTDOWN()) {
                        fbuf[i] = LETTER(final_message[i]);
                    }
                    break;
                }
            }
        }
    }
}
void display_refresh_fbuf(void) {
    unsigned char i;
    for(i = 0; i < 16; i++) {
        if(bits & (1 << i)) {
            fbuf[i] = LETTER(final_message[i]);
        }
    }
}


